 
package xyz.veronie.bgg.ui.parts;

import java.util.List;

import javax.annotation.PostConstruct;
import javax.inject.Inject;

import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.UIEventTopic;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.e4.ui.workbench.modeling.EPartService.PartState;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.wb.swt.ResourceManager;
import org.eclipse.wb.swt.SWTResourceManager;

import xyz.veronie.bgg.result.Thing;
import xyz.veronie.bgg.result.ThingProvider;
import xyz.veronie.bgg.types.EventConstants;
import xyz.veronie.bgg.ui.handlers.HandleSaveGamelist;
import xyz.veronie.bgg.ui.helpers.BatColors;
import xyz.veronie.bgg.ui.helpers.BatLayouts;

public class BatMain {
	private Table tableGameList;
	private TableViewer tableViewer;
	
	@Inject
	private ThingProvider thingProvider;
	
	@Inject 
	private IEventBroker eventBroker;
	
	@Inject
	public BatMain() {
		
	}
	
	@PostConstruct
	public void postConstruct(Composite parent, MApplication application, EPartService partService, EModelService modelService) {
		Color bgColor = BatColors.getBackgroundColor();
		
		parent.setBackground(bgColor);
		GridLayout gl_parent = new GridLayout(1, false);
		gl_parent.horizontalSpacing = 0;
		gl_parent.verticalSpacing = 0;
		gl_parent.marginWidth = 0;
		gl_parent.marginHeight = 0;
		parent.setLayout(gl_parent);
		
		Composite main = new Composite(parent, SWT.NONE);
		GridData gd_main = new GridData(SWT.LEFT, SWT.TOP, true, true, 1, 1);
		gd_main.minimumHeight = 192;
		gd_main.minimumWidth = 348;
		main.setLayoutData(gd_main);
		main.setBackground(bgColor);
		GridLayout gl_main = new GridLayout(1, false);
		gl_main.verticalSpacing = 24;
		BatLayouts.applyStandardSpacing(gl_main);
		main.setLayout(gl_main);
		
		Composite buttonRow = new Composite(main, SWT.NONE);
		buttonRow.setBackground(bgColor);
		GridLayout gl_buttonRow = new GridLayout(5, false);
		gl_buttonRow.verticalSpacing = 8;
		gl_buttonRow.marginWidth = 0;
		gl_buttonRow.marginHeight = 8;
		gl_buttonRow.horizontalSpacing = 8;
		BatLayouts.applyStandardSpacing(gl_buttonRow);
		buttonRow.setLayout(gl_buttonRow);
		buttonRow.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, true, false, 1, 1));
		
		Button btnFetch = new Button(buttonRow, SWT.NONE);
		btnFetch.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				List<MPart> parts = modelService.findElements(application, "xyz.veronie.bgg.ui.part.fetch", MPart.class);
				if(parts != null && parts.size() >= 1) {
					MPart fetchPart = parts.get(0);
					partService.showPart(fetchPart, PartState.CREATE);
					partService.showPart(fetchPart, PartState.ACTIVATE);
				}
			}
		});
		
		btnFetch.setToolTipText("Fetch games from BGG");
		btnFetch.setForeground(SWTResourceManager.getColor(SWT.COLOR_TRANSPARENT));
		btnFetch.setBackground(SWTResourceManager.getColor(SWT.COLOR_TRANSPARENT));
		btnFetch.setImage(ResourceManager.getPluginImage("xyz.veronie.bgg.ui", "icons/noun_Download_60x60.png"));
		
		Button btnSave = new Button(buttonRow, SWT.NONE);
		btnSave.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				// TODO: find a loose coupling instead. Where is EHandlerService???
				HandleSaveGamelist handleSaveGameList = new HandleSaveGamelist(thingProvider);
				if(handleSaveGameList.canExecute()) {
					handleSaveGameList.execute(parent.getShell());
				}
			}
		});
		btnSave.setToolTipText("Save list of games");
		btnSave.setBackground(SWTResourceManager.getColor(SWT.COLOR_TRANSPARENT));
		btnSave.setForeground(SWTResourceManager.getColor(SWT.COLOR_TRANSPARENT));
		btnSave.setImage(ResourceManager.getPluginImage("xyz.veronie.bgg.ui", "icons/noun_Save_60x60.png"));
		
		Button btnLoad = new Button(buttonRow, SWT.NONE);
		btnLoad.setBackground(SWTResourceManager.getColor(SWT.COLOR_TRANSPARENT));
		btnLoad.setImage(ResourceManager.getPluginImage("xyz.veronie.bgg.ui", "icons/noun_open_60x60.png"));
		btnLoad.setToolTipText("Load list of games");
		
		Button btnUndo = new Button(buttonRow, SWT.NONE);
		btnUndo.setToolTipText("Undo game list operation");
		btnUndo.setImage(ResourceManager.getPluginImage("xyz.veronie.bgg.ui", "icons/noun_Undo_60x60.png"));
		btnUndo.setBackground(SWTResourceManager.getColor(SWT.COLOR_TRANSPARENT));
		
		Button btnExport = new Button(buttonRow, SWT.NONE);
		btnExport.setBackground(SWTResourceManager.getColor(SWT.COLOR_TRANSPARENT));
		btnExport.setImage(ResourceManager.getPluginImage("xyz.veronie.bgg.ui", "icons/export_nandeck_60x60.png"));
		btnExport.setToolTipText("Export for nanDeck");
		
		tableViewer = new TableViewer(main, SWT.BORDER | SWT.FULL_SELECTION | SWT.MULTI);
		tableGameList = tableViewer.getTable();
		tableGameList.setLinesVisible(true);
		tableGameList.setHeaderVisible(true);
		GridData gd_tableGameList = new GridData(SWT.LEFT, SWT.TOP, true, true, 1, 1);
		gd_tableGameList.heightHint = 466;
		tableGameList.setLayoutData(gd_tableGameList);
		
		createColumns(tableViewer);
		tableViewer.setContentProvider(new ArrayContentProvider());
        // Get the content for the viewer, setInput will call getElements in the
        // contentProvider
		tableViewer.setInput(thingProvider.getThings());
        // make the selection available to other views
        // TODO: getSite().setSelectionProvider(viewer);
        // Set the sorter for the table
		tableViewer.addSelectionChangedListener(new ISelectionChangedListener() {
            @Override
            public void selectionChanged(SelectionChangedEvent event) {
                IStructuredSelection selection = tableViewer.getStructuredSelection();
                Thing thing = (Thing)selection.getFirstElement();
                eventBroker.send(EventConstants.TOPIC_THING_SELECTION, thing);
            }
        });
        
        
		Composite statusRow = new Composite(main, SWT.NONE);
		statusRow.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, true, false, 1, 1));
		GridLayout gl_statusRow = new GridLayout(1, false);
		gl_statusRow.verticalSpacing = 8;
		gl_statusRow.marginWidth = 8;
		gl_statusRow.marginHeight = 8;
		gl_statusRow.horizontalSpacing = 8;
		BatLayouts.applyStandardSpacing(gl_statusRow);
		statusRow.setLayout(gl_statusRow);
		
		Label lblResultStatus = new Label(statusRow, SWT.NONE);
		lblResultStatus.setText("0 items");
		
	}


	
	
	
	
    public TableViewer getViewer() {
        return tableViewer;
    }
	
	// This will create the columns for the table
    private void createColumns(final TableViewer viewer) {
    	
    	// https://stackoverflow.com/questions/12641354/putting-an-image-in-to-a-jface-table-cell-is-causing-gap-for-image-to-appear-in
    	TableViewerColumn col = createTableViewerColumn(Thing.IdHeader, 250, 0);
    	col.setLabelProvider(new ColumnLabelProvider() {
            @Override
            public String getText(Object element) {
                Thing t = (Thing) element;
                return t.getField((int)col.getColumn().getData());
            }
        });
    	
    	TableViewerColumn col2 = createTableViewerColumn(Thing.ThumbHeader, 150, 1);
    	col2.setLabelProvider(new ColumnLabelProvider() {
    		@Override
    		public Image getImage(Object element) {
    			Thing t = (Thing) element;
    			Image img = t.getThumbnail();
    			return img;
    		}
    		
    		@Override
            public String getText(Object element) {
    			return "";
    		}
    		
        });
    	
    	TableViewerColumn col3 = createTableViewerColumn(Thing.NameHeader, 400, 2);
    	col3.setLabelProvider(new ColumnLabelProvider() {
            @Override
            public String getText(Object element) {
                Thing t = (Thing) element;
                return t.getField((int)col3.getColumn().getData());
            }
        });
    }

    private TableViewerColumn createTableViewerColumn(String title, int bound, final int colNumber) {
        final TableViewerColumn viewerColumn = new TableViewerColumn(tableViewer,
                SWT.NONE);
        final TableColumn column = viewerColumn.getColumn();
        column.setText(title);
        column.setWidth(bound);
        column.setAlignment(SWT.LEFT);
        column.setResizable(true);
        column.setMoveable(true);
        column.setData(colNumber);
        
        return viewerColumn;

    }
    
    
    

    /**
     * Passing the focus request to the viewer's control.
     */
    public void setFocus() {
    	tableViewer.getControl().setFocus();
    }
    
    
    @Inject
	@Optional
	private void subscribeTopicResultChanged
	    (@UIEventTopic(EventConstants.TOPIC_RESULT_CHANGED)
	        String empty) {
    	System.out.println("TOPIC_RESULT_CHANGED");
    	List<Thing> things = thingProvider.getThings();
    	tableViewer.setInput(things);
    	tableViewer.refresh(true);
//    	if(things != null) {
//    		statsLabel.setText(Integer.toString(things.size()) + " items");
//    	    statsLabel.redraw();
//    	}
	}

}